; #INDEX# =======================================================================================================================
; Nom .........: Translate
; Version .....: 1.0.2
; AutoIt Version : 3.3.6++
; Language ......: Franais
; Description ...: Fonction pour traduire du texte via Google Translate.
; Auteur(s) .....: aurel2108 (aurel2108 at gmail dot com)
;
; Version 1.0 : 22/07/2011
;			- Premire version
;
; Version 1.0.1 : 22/07/2011
; 			- Fin de l'utilisation de inet.au3 et _InetGetSource()
;			- Utilisation de InetRead() et BinaryToString()
;
; Version 1.0.2 : 28/07/2011
; 			- Fin de l'utilisation de InetRead() et BinaryToString()
;			- Utilisation du protocole TCP
;			- Rapidit d'excution (16x moins de temps)
; ===============================================================================================================================

#include <Array.au3>
#Include <Timers.au3>

Dim Const $langs[32] = ['fr', 'en', 'de', 'es', 'it', 'pt', 'ca', 'hr', 'da', 'et', 'fi', 'gl', 'cy', 'hu', 'id', 'ga', 'is', 'lt','ms', 'mt', 'nl', 'no', 'pl', 'ro', 'sk', 'sl', 'sv', 'sw', 'tl', 'ts', 'tr', 'vi']

#cs
;Exemple (Traduit 'hello' en franais et l'affiche dans une MsgBox) :

$timer = _Timer_Init()
$translation = _Translate('hello', 'fr')
MsgBox(32, 'Translate', 'Temps d''excution : ' & Round(_Timer_Diff($timer)) & 'ms' & @CRLF & 'Texte traduit : ' & $translation)
#ce

; #FUNCTION# ====================================================================================================================
; Nom...........: _Translate
; Description ...: Traduire un texte en utilisant Google Translate.
; Syntaxe.........: _Translate($text, $outputLang[, $inputLang])
; Paramtres ....: $text - Texte  traduire
;                  $outputLang  - Langue de sortie
;				   $inputLang - [optional] Langue d'entre (langue du texte)
; Valeurs de retour .: Succs - Retourne le texte traduit
;                 	   chec  - -1
; Auteur ........: aurel2108
; Remarques .......: Si le code source de Google Translate change, la fonction pourrait ne plus fonctionner, si c'est le cas, veuillez le signaler  l'auteur. ;)
;				   $inputLang et $outputLang doivent tre les "initiales" de la langue tel que 'fr' pour le franais, 'en' pour l'anglais, etc...
; Exemple .......: Oui (ci-dessus)
; ===============================================================================================================================

Dim $resultat[2]

func _Translate($text, $outputLang, $inputLang = 'auto')
	if _ArraySearch($langs, $outputLang) <> -1 Then
		if _ArraySearch($langs, $inputLang) == -1 Then
			$inputLang = 'auto'
		EndIf
		$source = _HTTPGet("translate.google.com", "/?hl=&ie=UTF-8&text=" & $text & "&sl=" & $inputLang & "&tl=" & $outputLang)
		$resultat = StringRegExp($source,'onmouseout="this.style.backgroundColor=(.*?)>(.*?)</span>',2)
		return $resultat[2]
	Else
		return -1
	EndIf
EndFunc

;Internals functions

Func _HTTPGet($sHost, $sPage)
    Local $iSocket = _HTTPConnect($sHost)
    If @error Then Return SetError(1, 0, "")

    Local $sCommand = "GET " & $sPage & " HTTP/1.1" & @CRLF

    $sCommand &= "Host: " & $sHost & @CRLF
    $sCommand &= "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0" & @CRLF
    $sCommand &= "Referer: " & $sHost & @CRLF
    $sCommand &= "Connection: close" & @CRLF & @CRLF

    Local $BytesSent = TCPSend($iSocket, $sCommand)
    If $BytesSent = 0 Then Return SetError(2, @error, 0)

    Local $sRecv = "", $sCurrentRecv

    While 1
        $sCurrentRecv = TCPRecv($iSocket, 16)
        If @error <> 0 Then ExitLoop
        If $sCurrentRecv <> "" Then $sRecv &= $sCurrentRecv
    WEnd

    _HTTPShutdown($iSocket)

    Return $sRecv
EndFunc

Func _HTTPConnect($sHost, $iPort=80)
    TCPStartup()

    Local $sName_To_IP = TCPNameToIP($sHost)
    Local $iSocket = TCPConnect($sName_To_IP, $iPort)

    If $iSocket = -1 Then
        TCPCloseSocket($iSocket)
        Return SetError(1, 0, "")
    EndIf

    Return $iSocket
EndFunc

Func _HTTPShutdown($iSocket)
    TCPCloseSocket($iSocket)
    TCPShutdown()
EndFunc